mod admin;
mod pastes_public;

use std::{borrow::Cow, time::SystemTime};

use axum::Router;
use chrono::{DateTime, Utc};
use maud::{html, Markup, DOCTYPE};

use super::SharedState;

pub fn router() -> Router<SharedState> {
    Router::new()
        .nest("/p", pastes_public::router())
        .nest("/admin", admin::router())
}

const BC_INDEX: Breadcrumb = Breadcrumb::new_static("NCPN", "/");

struct Breadcrumb<'a> {
    title: Cow<'a, str>,
    href: Cow<'a, str>,
}

impl<'a> Breadcrumb<'a> {
    fn new(title: impl Into<Cow<'a, str>>, href: impl Into<Cow<'a, str>>) -> Self {
        Breadcrumb {
            title: title.into(),
            href: href.into(),
        }
    }
}

impl Breadcrumb<'static> {
    const fn new_static(title: &'static str, href: &'static str) -> Self {
        Breadcrumb {
            title: Cow::Borrowed(title),
            href: Cow::Borrowed(href),
        }
    }
}

fn page(title: &str, breadcrumbs: &[Breadcrumb], content: Markup) -> Markup {
    html! {
        (DOCTYPE)
        html {
            head {
                meta charset="utf-8";
                title { (format!("{title} | NCPN")) }
                link rel="stylesheet" href="/style.css";
                script src="/script.js" defer {}
            }
            body {
                header {
                    div {
                        @for b in breadcrumbs {
                            a.tl href=(b.href) { (b.title) }
                            span { "/" }
                        }
                    }
                    h1 { (title) }
                }
                (content)
                footer {
                    "© 2024 mrrp meow :3"
                }
            }
        }
    }
}

fn dt_iso(t: SystemTime) -> Markup {
    let dt: DateTime<Utc> = t.into();
    let iso = dt.to_rfc3339();
    html! {
        time datetime=(iso) { (iso) }
    }
}
