// 01HZT59RTH4R6P1TYE6NMAFYEP 018FF454-E351-260D-60EB-CE3568A7F9D6

use askama_axum::Template;
use axum::{
    extract::{Path, State},
    routing::get,
    Router,
};
use sqlx::query;
use ulid::Ulid;
use uuid::Uuid;

use crate::{app::SharedState, error::AppError};

pub(super) fn router() -> Router<SharedState> {
    Router::new()
        .route("/:id", get(show_paste))
        .route("/:id/raw", get(show_paste_raw))
}

#[derive(Template)]
#[template(path = "paste.html")]
struct PasteTemplate {
    id: Ulid,
    title: String,
    content: String,
}

async fn show_paste(
    Path(id): Path<Ulid>,
    State(SharedState { db, .. }): State<SharedState>,
) -> Result<PasteTemplate, AppError> {
    match query!(
        "SELECT title, content FROM paste WHERE id = $1",
        Uuid::from(id),
    )
    .fetch_optional(&db)
    .await?
    {
        Some(r) => Ok(PasteTemplate {
            id,
            title: r.title,
            content: r.content,
        }),
        None => Err(AppError::PasteNotFound(id)),
    }
}

async fn show_paste_raw(
    Path(id): Path<Ulid>,
    State(SharedState { db, .. }): State<SharedState>,
) -> Result<String, AppError> {
    match query!("SELECT content FROM paste WHERE id = $1", Uuid::from(id))
        .fetch_optional(&db)
        .await?
    {
        Some(r) => Ok(r.content),
        None => Err(AppError::PasteNotFound(id)),
    }
}
