use std::{net::SocketAddr, path::PathBuf};

use serde::Deserialize;
use url::Url;

#[derive(Clone, Debug, Deserialize)]
pub struct Config {
    #[serde(default = "default_listen_addr")]
    pub listen_addr: SocketAddr,

    #[serde(default = "default_db_socket")]
    pub db_socket: PathBuf,
    #[serde(default = "default_db_username")]
    pub db_username: String,
    #[serde(default = "default_db_database")]
    pub db_database: String,

    #[serde(default = "default2_destination")]
    pub default_destination: Url,

    #[serde(default = "default_file_store_dir")]
    pub file_store_dir: PathBuf,
    #[serde(default = "default_file_temp_dir")]
    pub file_temp_dir: PathBuf,
}

fn default_listen_addr() -> SocketAddr {
    ([0, 0, 0, 0], 3000).into()
}

fn default_db_socket() -> PathBuf {
    "/var/run/postgresql/".into()
}

fn default_db_username() -> String {
    "ncpn".into()
}

fn default_db_database() -> String {
    default_db_username()
}

fn default2_destination() -> Url {
    "https://goob.cc/r".parse().expect("hardcoded URL is valid")
}

fn default_file_store_dir() -> PathBuf {
    PathBuf::from("files")
}

fn default_file_temp_dir() -> PathBuf {
    PathBuf::from("temp")
}
